Struct CreateOptions

pub struct CreateOptions {
    pub page_size: PageSize,
    pub max_streams: u32,
}
Options for creating a new PDB/MSF file.

Fields

page_size: PageSize

The page size to use. This must be in the range [MIN_PAGE_SIZE..=MAX_PAGE_SIZE].

max_streams: u32

The maximum number of streams that we will allow to be created using new_stream or nil_stream. The default value is 0xfffe, which prevents overflowing the 16-bit stream indexes that are used in PDB (or confusing them with the “nil” stream index).

Applications may increase this value beyond the default, but this will produce MSF files that are not usable by most PDB tools.