pub struct StreamWriter<'a, F> { /* private fields */ }Expand description
Provides read/write access for a stream within an MSF (PDB) file.
Implementations
Sourceimpl<'a, F> StreamWriter<'a, F>
impl<'a, F> StreamWriter<'a, F>
Sourcepub fn len(&self) -> u32
pub fn len(&self) -> u32
The length in bytes of this stream. This will return zero for nil streams.
Sourcepub fn set_contents(&mut self, data: &[u8]) -> Result<()>
pub fn set_contents(&mut self, data: &[u8]) -> Result<()>
Replaces the contents of a stream. The length of the stream is set to the length of data.
Sourcepub fn write_at_mut(&mut self, buf: &[u8], offset: u64) -> Result<usize>
pub fn write_at_mut(&mut self, buf: &[u8], offset: u64) -> Result<usize>
Writes data to a given position in the file.
This method exists to work around the limitation of WriteAt, which takes &self
instead of &mut self.
Sourcepub fn write_all_at_mut(&mut self, buf: &[u8], offset: u64) -> Result<()>
pub fn write_all_at_mut(&mut self, buf: &[u8], offset: u64) -> Result<()>
Writes data to a given position in the file.
This method exists to work around the limitation of WriteAt, which takes &self
instead of &mut self.
Sourcepub fn set_len(&mut self, len: u32) -> Result<()>
pub fn set_len(&mut self, len: u32) -> Result<()>
Sets the length of this stream, in bytes.
If the stream is a nil stream, this changes it to a non-nil stream, even if len is zero.
If you are writing data to a stream, try to avoid using set_len to preallocate storage.
It will allocate pages (yay) but it will waste I/O time by writing zero-fill data to
all of the pages. There is currently no optimized path for preallocating pages that relies
on the underlying storage provide zero-filling pages.
If you are replacing the contents of a large stream, just write everything from page zero
and then call set_len at the end of the write.
Sourcepub fn into_random(self) -> RandomStreamWriter<'a, F>
pub fn into_random(self) -> RandomStreamWriter<'a, F>
Converts this StreamWriter into a RandomStreamWriter.